from rest_framework import serializers
from issuestracker.models import Position, Candidate

from .Candidate import CandidateSerializer


class PositionSerializer(serializers.ModelSerializer):
    candidates = CandidateSerializer(many=True, read_only=True)

    def update_candidates(self, instance, candidates):
        candidate_ids = [c["id"] for c in candidates]
        # Remove candidates that were removed
        for c in instance.candidates.all():
            if c.id not in candidate_ids:
                instance.candidates.remove(c)

        # Add candidates that were added
        for c in candidates:
            try:
                instance.candidates.get(id=c["id"])
            except Candidate.DoesNotExist:
                instance.candidates.add(Candidate.objects.get(id=c["id"]))

    def create(self, validated_data):
        candidates = validated_data.pop("candidates", [])
        p = Position(**validated_data)
        p.save()
        self.update_candidates(p, candidates)
        return p

    def update(self, instance, validated_data):
        candidates = validated_data.get("candidates", [])
        self.update_candidates(instance, candidates)

        instance.name = validated_data.get("name", instance.name)
        instance.explanation = validated_data.get(
            "explanation", instance.explanation
        )
        instance.issue = validated_data.get("issue", instance.issue)
        instance.order = validated_data.get("order", instance.order)
        instance.save()
        return instance

    class Meta:
        model = Position
        fields = "__all__"


class PositionSimpleSerializer(serializers.ModelSerializer):
    class Meta:
        model = Position
        fields = ("name", "explanation")



from .base import BakeryBase
from rest_framework.response import Response
from issuestracker.models import Category, Issue, Candidate
from issuestracker.serializers.bakery.SiteMap import (
    CategorySerializer,
    IssueSerializer,
    CandidateSerializer,
)


class BakerySitemap(BakeryBase):
    def get(self, request, format=None):
        return Response(
            {
                "category": [
                    d["slug"]
                    for d in CategorySerializer(
                        Category.live.all(), many=True
                    ).data
                ],
                "issue": [
                    d["slug"]
                    for d in IssueSerializer(Issue.live.all(), many=True).data
                ],
                "candidate": [
                    d["slug"]
                    for d in CandidateSerializer(
                        Candidate.objects.all(), many=True
                    ).data
                ],
            }
        )



from django.db import models


class Update(models.Model):
    """
    An update about an issue (e.g. "Cory Booker now supports X.")
    """

    text = models.TextField()
    issue = models.ForeignKey("Issue", on_delete=models.PROTECT)
    created_on = models.DateField(auto_now_add=True)

    def __str__(self):
        return self.question

    class Meta:
        ordering = ["-created_on"]



# Generated by Django 2.2 on 2019-04-29 14:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0003_auto_20190429_1422'),
    ]

    operations = [
        migrations.AlterField(
            model_name='candidate',
            name='slug',
            field=models.SlugField(default='', unique=True),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-01 18:52

from django.db import migrations, models
import issuestracker.utils.storage_service


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0008_remove_candidate_headshot'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='social_description',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='category',
            name='social_image',
            field=models.ImageField(blank=True, null=True, storage=issuestracker.utils.storage_service.StorageService(), upload_to='%Y/%m/'),
        ),
        migrations.AddField(
            model_name='category',
            name='social_title',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='issue',
            name='social_description',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='issue',
            name='social_image',
            field=models.ImageField(blank=True, null=True, storage=issuestracker.utils.storage_service.StorageService(), upload_to='%Y/%m/'),
        ),
        migrations.AddField(
            model_name='issue',
            name='social_title',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]



# Generated by Django 2.2 on 2019-04-16 21:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Candidate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uid', models.CharField(max_length=250, unique=True)),
                ('fec_uid', models.CharField(blank=True, max_length=50, null=True, unique=True)),
                ('name', models.CharField(max_length=100)),
                ('headshot', models.URLField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True)),
                ('slug', models.SlugField(unique=True)),
                ('summary', models.TextField(blank=True, null=True)),
                ('icon', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Issue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=100, null=True, unique=True)),
                ('slug', models.SlugField(unique=True)),
                ('summary', models.TextField(blank=True, null=True)),
                ('order', models.IntegerField()),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='issuestracker.Category')),
            ],
        ),
        migrations.CreateModel(
            name='Update',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('created_on', models.DateField(auto_now_add=True)),
                ('issue', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='issuestracker.Issue')),
            ],
            options={
                'ordering': ['-created_on'],
            },
        ),
        migrations.CreateModel(
            name='Story',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('headline', models.CharField(max_length=200)),
                ('link', models.URLField()),
                ('order', models.IntegerField()),
                ('issue', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='issuestracker.Issue')),
            ],
            options={
                'ordering': ['order'],
            },
        ),
        migrations.CreateModel(
            name='Position',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('explanation', models.TextField(blank=True, null=True)),
                ('order', models.IntegerField()),
                ('candidates', models.ManyToManyField(to='issuestracker.Candidate')),
                ('issue', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='issuestracker.Issue')),
            ],
            options={
                'ordering': ['order'],
                'unique_together': {('name', 'issue')},
            },
        ),
        migrations.CreateModel(
            name='Faq',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('question', models.CharField(max_length=500)),
                ('answer', models.TextField(blank=True, null=True)),
                ('order', models.IntegerField()),
                ('issue', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='issuestracker.Issue')),
            ],
            options={
                'ordering': ['order'],
                'unique_together': {('question', 'issue')},
            },
        ),
    ]



import logging
import os

from issuestracker.utils.aws import defaults, get_bucket
from celery import shared_task

logger = logging.getLogger('tasks')

OUTPUT_PATH = 'issuestracker/data/'


@shared_task(acks_late=True)
def publish_to_aws(pk):
    key = os.path.join(
        OUTPUT_PATH,
        'data.json'
    )

    bucket = get_bucket()

    bucket.put_object(
        Key=key,
        ACL=defaults.ACL,
        Body='{}',
        CacheControl=defaults.CACHE_HEADER,
        ContentType='application/json'
    )

    logger.info('Published to AWS')



from .base import Base


class Category(Base):
    template_name = "issuestracker/category.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["category"] = kwargs["category"]
        return context



# Generated by Django 2.2 on 2019-04-29 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0002_auto_20190424_1640'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='issue',
            options={'ordering': ['name']},
        ),
        migrations.AddField(
            model_name='candidate',
            name='slug',
            field=models.SlugField(blank=True, null=True),
        ),
    ]



from rest_framework import serializers
from issuestracker.models import Category, Issue


class CandidateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = ("slug",)


class IssueSerializer(serializers.ModelSerializer):
    class Meta:
        model = Issue
        fields = ("slug", "name")


class CategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = ("slug", "name")


class CombinedSerializer(serializers.ModelSerializer):
    issue_set = serializers.SerializerMethodField()

    def get_issue_set(self, obj):
        return IssueSerializer(
            obj.issue_set.filter(status="live"), many=True, read_only=True
        ).data

    class Meta:
        model = Category
        fields = ("slug", "name", "icon", "issue_set")



from django.urls import path, include
from rest_framework import routers
from .viewsets import CategoryViewset, IssueViewset, CandidateViewset
from .views import (
    Home,
    Category,
    Issue,
    BakeryHome,
    BakeryCategory,
    BakeryIssue,
    BakeryCandidate,
    BakerySitemap,
)

# Django REST
router = routers.DefaultRouter()
router.register(r"category", CategoryViewset, base_name="category")
router.register(r"issue", IssueViewset, base_name="issue")
router.register(r"candidate", CandidateViewset, base_name="candidate")

urlpatterns = [
    path("api/", include(router.urls)),
    path(
        "api/bakery/sitemap/", BakerySitemap.as_view(), name="bakery-sitemap"
    ),
    path("api/bakery/home/", BakeryHome.as_view(), name="bakery-home"),
    path(
        "api/bakery/category/<slug:category>/",
        BakeryCategory.as_view(),
        name="bakery-category",
    ),
    path(
        "api/bakery/issue/<slug:issue>/",
        BakeryIssue.as_view(),
        name="bakery-issue",
    ),
    path(
        "api/bakery/candidate/<slug:candidate>/",
        BakeryCandidate.as_view(),
        name="bakery-candidate",
    ),
    path("", Home.as_view(), name="home"),
    path("category/<slug:category>/", Category.as_view(), name="category"),
    path("issue/<slug:issue>/", Issue.as_view(), name="issue"),
]



from django.db import models
from issuestracker.managers.live import LiveManager
from issuestracker.utils.storage_service import StorageService


class Issue(models.Model):
    """
    A specific issue of a category (e.g. "Medicare 4 All", "Carbon Tax", etc.)
    """

    name = models.CharField(max_length=100, null=True, blank=True, unique=True)
    slug = models.SlugField(max_length=50, unique=True)
    dek = models.TextField(null=True, blank=True)
    summary = models.TextField(null=True, blank=True)
    category = models.ForeignKey("Category", on_delete=models.PROTECT)
    last_updated = models.DateTimeField(auto_now=True)
    last_published = models.DateTimeField(null=True, blank=True)
    status = models.CharField(
        max_length=5,
        choices=(("draft", "Draft"), ("live", "Live")),
        default="draft",
    )

    objects = models.Manager()
    live = LiveManager()

    social_title = models.CharField(max_length=100, null=True, blank=True)
    social_description = models.TextField(null=True, blank=True)
    social_image = models.ImageField(
        upload_to="%Y/%m/", storage=StorageService(), null=True, blank=True
    )

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]



from .base import BakeryBase
from rest_framework.response import Response
from issuestracker.models import Category, Candidate, Issue, Story
from issuestracker.serializers.bakery.SiteMap import CombinedSerializer
from issuestracker.serializers.bakery.HomePage import (
    CategorySerializer,
    CandidateSerializer,
    StorySerializer,
)


class BakeryHome(BakeryBase):
    def get(self, request, format=None):
        last_updated_category = (
            Category.objects.order_by("-last_published").first().last_published
        )
        last_updated_issue = (
            Issue.objects.order_by("-last_published").first().last_published
        )

        last_updated = self.get_last_updated(
            last_updated_category, last_updated_issue
        )

        return Response(
            {
                "last_updated": last_updated,
                "categories": CategorySerializer(
                    Category.live.all(), many=True
                ).data,
                "candidates": CandidateSerializer(
                    Candidate.objects.all(), many=True
                ).data,
                "latest_coverage": StorySerializer(
                    Story.objects.order_by("-date_added"), many=True
                ).data[0:3],
                "sitemap": CombinedSerializer(
                    Category.live.all(), many=True
                ).data,
            }
        )



from storages.backends.s3boto3 import S3Boto3Storage
from issuestracker.conf import settings


class StorageService(S3Boto3Storage):
    bucket_name = settings.AWS_S3_BUCKET
    access_key = settings.AWS_ACCESS_KEY_ID
    secret_key = settings.AWS_SECRET_ACCESS_KEY
    file_overwrite = False
    querystring_auth = False
    object_parameters = {"CacheControl": "max-age=86400", "ACL": "public-read"}
    custom_domain = settings.CLOUDFRONT_ALTERNATE_DOMAIN
    location = settings.S3_UPLOAD_ROOT



"""
Use this file to configure pluggable app settings and resolve defaults
with any overrides set in project settings.
"""

from django.conf import settings as project_settings

from .exceptions import IssuestrackerConfigError


class Settings:
    pass


Settings.AUTH_DECORATOR = getattr(
    project_settings,
    'ISSUESTRACKER_AUTH_DECORATOR',
    'django.contrib.auth.decorators.login_required'
)

Settings.SECRET_KEY = getattr(
    project_settings, 'ISSUESTRACKER_SECRET_KEY', 'a-bad-secret-key')

Settings.AWS_ACCESS_KEY_ID = getattr(
    project_settings, 'ISSUESTRACKER_AWS_ACCESS_KEY_ID', None)

Settings.AWS_SECRET_ACCESS_KEY = getattr(
    project_settings, 'ISSUESTRACKER_AWS_SECRET_ACCESS_KEY', None)

Settings.AWS_REGION = getattr(
    project_settings, 'ISSUESTRACKER_AWS_REGION', None)

Settings.AWS_S3_BUCKET = getattr(
    project_settings, 'ISSUESTRACKER_AWS_S3_BUCKET', None)

Settings.CLOUDFRONT_ALTERNATE_DOMAIN = getattr(
    project_settings, 'ISSUESTRACKER_CLOUDFRONT_ALTERNATE_DOMAIN', None)

Settings.S3_UPLOAD_ROOT = getattr(
    project_settings, 'ISSUESTRACKER_S3_UPLOAD_ROOT', 'uploads/issuestracker')


settings = Settings



import logging
from .base import TokenAuthedViewSet
from django.utils import timezone
from rest_framework import status
from rest_framework.response import Response
from rest_framework.decorators import action
from issuestracker.utils.update_fk_models import update_fk_models
from issuestracker.models import Issue, Position, Faq, Story
from issuestracker.serializers import (
    IssueSerializer,
    IssueListSerializer,
    PositionSerializer,
    FaqSerializer,
    StorySerializer,
)

logger = logging.getLogger("serializer")
tasks_logger = logging.getLogger("tasks")


class IssueViewset(TokenAuthedViewSet):
    queryset = Issue.objects.all()
    lookup_field = "slug"
    pagination_class = None
    throttle_classes = []

    def get_serializer_class(self):
        if hasattr(self, "action") and self.action == "list":
            return IssueListSerializer
        return IssueSerializer

    def update(self, request, *args, **kwargs):
        id = request.data.pop("id")
        story_data = request.data.pop("story_set")
        faq_data = request.data.pop("faq_set")
        position_data = request.data.pop("position_set")
        instance = Issue.objects.get(pk=id)

        update_fk_models(instance, story_data, Story, StorySerializer)
        update_fk_models(instance, faq_data, Faq, FaqSerializer)
        update_fk_models(
            instance,
            position_data,
            Position,
            PositionSerializer,
            context_keys=["candidates"],
        )

        s = IssueSerializer(instance, data=request.data)

        if not s.is_valid():
            logger.error(s.errors)

        s.is_valid(raise_exception=True)

        s.save()

        return Response(s.data)

    def partial_update(self, request, *args, **kwargs):
        return Response(
            "Partial updates are not allowed at this endpoint.",
            status=status.HTTP_405_METHOD_NOT_ALLOWED,
        )

    @action(detail=True, methods=["post"])
    def publish(self, request, *args, **kwargs):
        slug = request.data.get("slug")

        update = self.update(request, *args, **kwargs)

        tasks_logger.info("Publish Too!")

        i = Issue.objects.get(slug=slug)
        i.last_published = timezone.now()
        i.save()

        return update



# Generated by Django 2.2 on 2019-04-24 16:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='last_published',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='category',
            name='last_updated',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name='issue',
            name='last_published',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='issue',
            name='last_updated',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



# flake8: noqa
from .Candidate import Candidate
from .Category import Category
from .Faq import Faq
from .Issue import Issue
from .Position import Position
from .Story import Story
from .Update import Update



from .base import Base


class Issue(Base):
    template_name = "issuestracker/issue.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["issue"] = kwargs["issue"]
        return context



from rest_framework import serializers
from issuestracker.models import Category, Issue, Candidate, Story
from django.db.models import Q


class StorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Story
        fields = ("headline", "link")


class CandidateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Candidate
        fields = ("uid",)


class IssueSerializer(serializers.ModelSerializer):
    position_count = serializers.SerializerMethodField()
    candidates_with_position_count = serializers.SerializerMethodField()

    def get_position_count(self, obj):
        return obj.position_set.all().count()

    def get_candidates_with_position_count(self, obj):
        count = 0
        for position in obj.position_set.all():
            count += position.candidates.count()

        return count

    class Meta:
        model = Issue
        fields = (
            "name",
            "slug",
            "dek",
            "position_count",
            "candidates_with_position_count",
        )


class CategorySerializer(serializers.ModelSerializer):
    issue_set = IssueSerializer(many=True, read_only=True)
    candidates_count = serializers.SerializerMethodField()
    story_set = serializers.SerializerMethodField()

    def get_candidates_count(self, obj):
        return Candidate.objects.all().count()

    def get_story_set(self, obj):
        stories = Story.objects.filter(Q(issue__category=obj))
        return StorySerializer(stories, many=True).data

    class Meta:
        model = Category
        fields = (
            "name",
            "slug",
            "icon",
            "summary",
            "candidates_count",
            "issue_set",
            "story_set",
        )



from rest_framework.viewsets import ModelViewSet, ReadOnlyModelViewSet
from rest_framework.permissions import IsAuthenticated
from issuestracker.utils.api_auth import TokenAPIAuthentication


class TokenAuthedViewSet(ModelViewSet):
    """
    ViewSet class that restricts views to our bots token.
    Also disables the default pagination.
    """

    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = (IsAuthenticated,)


class ReadOnlyTokenAuthedViewSet(ReadOnlyModelViewSet):
    """
    Read only ViewSet class that restricts views to our bots token.
    Also disables the default pagination.
    """

    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = (IsAuthenticated,)



# flake8: noqa
from .home import Home
from .category import Category
from .issue import Issue
from .bakery.home import BakeryHome
from .bakery.category import BakeryCategory
from .bakery.issue import BakeryIssue
from .bakery.candidate import BakeryCandidate
from .bakery.sitemap import BakerySitemap



# flake8: noqa
from .Candidate import CandidateSerializer
from .Category import CategorySerializer, CategoryListSerializer
from .Faq import FaqSerializer
from .Issue import IssueSerializer, IssueListSerializer
from .Position import PositionSerializer
from .Story import StorySerializer
from .Update import UpdateSerializer



from issuestracker.conf import settings
from django.conf import settings as project_settings
from django.contrib.auth.models import AnonymousUser
from django.utils.decorators import method_decorator
from rest_framework import authentication, exceptions


class TokenAPIAuthentication(authentication.BaseAuthentication):
    """
    DRF custom authentication class for viewsets.

    Uses app's secret key to authenticate AJAX requests.
    """
    def authenticate(self, request):
        # Don't enforce if DEBUG
        if project_settings.DEBUG:
            return (AnonymousUser, None)
        try:
            # Token should be prefixed with string literal "Token" plus
            # whitespace, e.g., "Token <TOKEN>".
            token = request.META.get('HTTP_AUTHORIZATION').split()[1]
        except:
            raise exceptions.AuthenticationFailed(
                'No token or incorrect token format')

        if token == settings.SECRET_KEY:
            return (AnonymousUser, None)
        raise exceptions.AuthenticationFailed('Unauthorized')



from issuestracker.utils.diff_relative_complement import diff
import logging


logger = logging.getLogger("serializer")


def update_fk_models(instance, data, model, serializer, context_keys=[]):
    """
    Given a model with instance names, array of dictionaries, a model,
    and a serializer: use the serializer's save method to update, create,
    or delete an entry.
    """
    # Delete models
    model_name = model.__name__.lower()
    old_model_ids = [
        m.id for m in getattr(instance, "{}_set".format(model_name)).all()
    ]
    new_model_ids = [m["id"] for m in data if "id" in m]
    for id in diff(old_model_ids, new_model_ids):
        model.objects.get(pk=id).delete()

    # Update/create models
    for row in data:
        id = row.pop("id", None)

        kwargs = {}
        for key in context_keys:
            kwargs[key] = row.pop(key)

        if id:
            m = model.objects.get(pk=id)
            s = serializer(m, data=row)
        else:
            s = serializer(data=row)

        if not s.is_valid():
            logger.error(s.errors)

        s.is_valid(raise_exception=True)

        s.save(**kwargs)



from rest_framework import serializers
from issuestracker.models import Category, Issue, Candidate, Position, Story


def get_position(obj, slug):
    candidate_slug = slug
    c = Candidate.objects.get(slug=candidate_slug)
    return c.position_set.get(issue=obj.id)


def get_candidates_with_position_count(obj):
    count = 0
    for position in obj.position_set.all():
        count += position.candidates.count()

    return count


class StorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Story
        fields = ("headline", "link")


class CandidateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Candidate
        fields = ("uid",)


class IssueSerializer(serializers.ModelSerializer):
    position = serializers.SerializerMethodField()
    explanation = serializers.SerializerMethodField()
    like_candidates = serializers.SerializerMethodField()

    def get_position(self, obj):
        try:
            return get_position(obj, self.context["slug"]).name
        except Position.DoesNotExist:
            return None

    def get_explanation(self, obj):
        try:
            return get_position(obj, self.context["slug"]).explanation
        except Position.DoesNotExist:
            return None

    def get_like_candidates(self, obj):
        candidate_slug = self.context["slug"]
        try:
            position = get_position(obj, self.context["slug"])
            return [
                cand.uid
                for cand in position.candidates.all()
                if cand.slug != candidate_slug
            ]
        except Position.DoesNotExist:
            return []

    class Meta:
        model = Issue
        fields = ("name", "slug", "position", "explanation", "like_candidates")


class CategorySerializer(serializers.ModelSerializer):
    issue_set = serializers.SerializerMethodField()

    def get_issue_set(self, obj):
        return IssueSerializer(
            self.context["issues"].filter(category=obj),
            many=True,
            read_only=True,
            context=self.context,
        ).data

    class Meta:
        model = Category
        fields = ("name", "slug", "icon", "issue_set")



from issuestracker.models import Category
from rest_framework import serializers
from .Issue import IssueListSerializer


class CategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = (
            "name",
            "slug",
            "icon",
            "summary",
            "status",
            "last_updated",
            "last_published",
        )


class CategoryListSerializer(serializers.ModelSerializer):
    issue_set = IssueListSerializer(many=True, read_only=True)

    class Meta:
        model = Category
        fields = ("id", "name", "summary", "slug", "icon", "issue_set")



from rest_framework import serializers
from issuestracker.models import Category, Issue, Candidate, Position, Story


class StorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Story
        fields = ("headline", "link")


class CandidateSerializer(serializers.ModelSerializer):
    view_count = serializers.SerializerMethodField()

    def get_view_count(self, obj):
        return Position.objects.filter(candidates=obj).count()

    class Meta:
        model = Candidate
        fields = ("uid", "view_count")


class IssueSerializer(serializers.ModelSerializer):
    class Meta:
        model = Issue
        fields = ("name", "slug")


class CategorySerializer(serializers.ModelSerializer):
    issue_set = IssueSerializer(many=True, read_only=True)

    class Meta:
        model = Category
        fields = ("name", "slug", "icon", "issue_set")



from .base import BakeryBase
from django.db.models import Q
from rest_framework.response import Response
from issuestracker.models import Category, Candidate, Story, Issue
from issuestracker.serializers.bakery.SiteMap import CombinedSerializer
from issuestracker.serializers.bakery.CandidatePage import (
    CategorySerializer,
    CandidateSerializer,
    StorySerializer,
)


class BakeryCandidate(BakeryBase):
    def get(self, request, candidate=None):
        selected_cand_query = Candidate.objects.filter(
            slug=self.kwargs["candidate"]
        )
        selected_cand = selected_cand_query.first()

        issues = Issue.live.filter(
            Q(position__candidates__in=selected_cand_query)
        )
        categories = Category.live.filter(Q(issue__in=issues))

        category_data = CategorySerializer(
            categories,
            context={"slug": candidate, "issues": issues},
            many=True,
        ).data

        last_updated_issue = (
            issues.order_by("-last_published").first().last_published
        )

        last_updated_category = (
            categories.order_by("-last_published").first().last_published
        )

        last_updated = self.get_last_updated(
            last_updated_category, last_updated_issue
        )

        return Response(
            {
                "uid": selected_cand.uid,
                "last_updated": last_updated.isoformat(" "),
                "categories": category_data,
                "candidates": CandidateSerializer(
                    Candidate.objects.all(), many=True
                ).data,
                "latest_coverage": StorySerializer(
                    Story.objects.order_by("-date_added"), many=True
                ).data[0:3],
                "sitemap": CombinedSerializer(
                    Category.live.all(), many=True
                ).data,
            }
        )



from django.db import models


class Faq(models.Model):
    """
    A question about a specific issue (e.g. "What is a Carbon Tax?")
    """

    question = models.CharField(max_length=500)
    answer = models.TextField(null=True, blank=True)
    issue = models.ForeignKey("Issue", on_delete=models.PROTECT)
    order = models.IntegerField()

    def __str__(self):
        return self.question

    class Meta:
        unique_together = ("question", "issue")
        ordering = ["order"]



from setuptools import find_packages, setup

setup(
    name="django-politico-issues-tracker",
    version="0.0.9",
    description="",
    url="https://github.com/The-Politico/django-politico-issues-tracker",
    author="POLITICO interactive news",
    author_email="interactives@politico.com",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Django",
        "Framework :: Django :: 2.0",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
    ],
    keywords="",
    packages=find_packages(exclude=["docs", "tests", "example"]),
    include_package_data=True,
    install_requires=[],
    extras_require={"test": ["pytest"]},
)



from .base import BakeryBase
from rest_framework.response import Response
from issuestracker.models import Issue, Candidate, Story, Category
from issuestracker.serializers.bakery.SiteMap import CombinedSerializer
from issuestracker.serializers.bakery.IssuePage import (
    IssueSerializer,
    CandidateSerializer,
    StorySerializer,
)


class BakeryIssue(BakeryBase):
    def get(self, request, issue=None):
        selected_issue = Issue.live.get(slug=self.kwargs["issue"])
        last_updated = selected_issue.last_published

        if last_updated is not None:
            last_updated = last_updated.isoformat(" ")

        return Response(
            {
                "last_updated": last_updated,
                "data": IssueSerializer(selected_issue).data,
                "candidates": CandidateSerializer(
                    Candidate.objects.all(), many=True
                ).data,
                "latest_coverage": StorySerializer(
                    Story.objects.order_by("-date_added"), many=True
                ).data[0:3],
                "sitemap": CombinedSerializer(
                    Category.live.all(), many=True
                ).data,
            }
        )



from issuestracker.models import Candidate
from rest_framework import serializers


class CandidateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Candidate
        fields = "__all__"



from rest_framework.views import APIView
from issuestracker.utils.api_auth import TokenAPIAuthentication
from issuestracker.models import Candidate
from issuestracker.serializers.Candidate import CandidateSerializer


class BakeryBase(APIView):
    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = []

    @staticmethod
    def get_last_updated(last_updated_category, last_updated_issue):
        if last_updated_category is None:
            if last_updated_issue is None:
                last_updated = None
            else:
                last_updated = last_updated_issue
        elif last_updated_issue is None:
            last_updated = last_updated_category
        else:
            last_updated = max(last_updated_category, last_updated_issue)

        if last_updated is not None:
            last_updated = last_updated.isoformat(" ")

        return last_updated

    def get_all_candidates(self):
        output = {}
        for candidate in Candidate.objects.all():
            output[candidate.slug] = CandidateSerializer(candidate).data
        return output
